@@ -1,6 +1,6 @@
 /*
-    A candidate replacement for Pspp's sheet
-    Copyright (C) 2016  John Darrington
+    A widget to display and manipulate tabular data
+    Copyright (C) 2016, 2017  John Darrington
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 #ifndef _SSW_AXIS_MODEL_H
 #define _SSW_AXIS_MODEL_H
 
-
+#include <glib-object.h>
 
 G_DECLARE_FINAL_TYPE (SswAxisModel, ssw_axis_model, SSW, AXIS_MODEL, GObject)
 
@@ -28,10 +28,10 @@ struct _SswAxisModel
 {
   GObject parent_instance;
 
-  gboolean wide2;
-  gboolean strike;
   guint size;
   gint offset;
+  void (*button_post_create_func) (GtkWidget *, guint, gpointer);
+  gpointer button_post_create_func_data;
 };
 
 struct _SswAxisModelClass
