@@ -105,35 +105,6 @@ limit_selection (SswSheetBody *body)
     priv->selection->end_y = ssw_sheet_axis_get_size (priv->vaxis) - 1;
 }
 
-
-void
-normalise_selection (const SswRange *in, SswRange *out)
-{
-  if (in->start_y < in->end_y)
-    {
-      out->end_y = in->end_y;
-      out->start_y = in->start_y;
-    }
-  else
-    {
-      out->start_y = in->end_y;
-      out->end_y = in->start_y;
-    }
-
-  if (in->start_x < in->end_x)
-    {
-      out->end_x = in->end_x;
-      out->start_x = in->start_x;
-    }
-  else
-    {
-      out->start_x = in->end_x;
-      out->end_x = in->start_x;
-    }
-}
-
-
-
 enum  {SELECTION_CHANGED,
        VALUE_CHANGED,
        n_SIGNALS};
@@ -145,8 +116,15 @@ get_active_cell (SswSheetBody *body, gint *col, gint *row)
 {
   PRIV_DECL (body);
 
-  *col = priv->selection->start_x;
-  *row = priv->selection->start_y;
+  SswSheetBody *active_body = NULL;
+
+  sscanf (priv->path, "r%dc%ds%p", row, col, &active_body);
+
+  if (active_body == NULL) /* There is no active cell! */
+    return FALSE;
+
+  if (active_body != body) /* This body is not active */
+    return FALSE;
 
   if (*col < 0 || *row < 0)
     return FALSE;
@@ -230,6 +208,32 @@ scroll_horizontally (SswSheetBody *body)
 
 
 
+static void
+normalise_selection (const SswRange *in, SswRange *out)
+{
+  if (in->start_y < in->end_y)
+    {
+      out->end_y = in->end_y;
+      out->start_y = in->start_y;
+    }
+  else
+    {
+      out->start_y = in->end_y;
+      out->end_y = in->start_y;
+    }
+
+  if (in->start_x < in->end_x)
+    {
+      out->end_x = in->end_x;
+      out->start_x = in->start_x;
+    }
+  else
+    {
+      out->start_x = in->end_x;
+      out->end_x = in->start_x;
+    }
+}
+
 static void
 draw_selection (SswSheetBody *body, cairo_t *cr)
 {
@@ -268,7 +272,7 @@ draw_selection (SswSheetBody *body, cairo_t *cr)
       if (0 < ssw_sheet_axis_find_boundary (priv->haxis, mySelect.start_x,
 					    &xpos_start,  NULL))
 	goto done;
-      
+
       if (0 < ssw_sheet_axis_find_boundary (priv->vaxis, mySelect.start_y,
 					    &ypos_start,  NULL))
 	goto done;
@@ -279,16 +283,16 @@ draw_selection (SswSheetBody *body, cairo_t *cr)
 					       &xextent);
       if (xrel < 0)
 	goto done;
-      
+
       gint xsize = (xrel == 0) ? xpos_end - xpos_start + xextent : width;
-      
+
 
       gint yrel = ssw_sheet_axis_find_boundary (priv->vaxis,
 						mySelect.end_y, &ypos_end,
 						&yextent);
       if (yrel < 0)
 	goto done;
-      
+
       gint ysize = (yrel == 0) ? ypos_end - ypos_start + yextent : height;
 
       gtk_render_background (sc, cr,
@@ -405,7 +409,7 @@ __draw (GtkWidget *widget, cairo_t *cr)
 
   if (yy == 0 && xx == 0 && priv->editor == NULL)
     start_editing (body, NULL);
-  
+
   if ((gtk_widget_is_focus (widget) ||
        (priv->editor && gtk_widget_is_focus (priv->editor))))
     gtk_css_provider_load_from_data (cp, focused_border, strlen (focused_border), 0);
@@ -1163,7 +1167,7 @@ set_editor_widget_value (SswSheetBody *body, GValue *value, GtkEditable *editabl
 
   /* Allow the datum to be changed only if the "editable" property is set */
   gtk_editable_set_editable (editable, priv->editable);
-  
+
   if (GTK_IS_SPIN_BUTTON (editable))
     {
       if (G_IS_VALUE (value))
@@ -1219,7 +1223,7 @@ on_data_change (GtkTreeModel *tm, guint posn, guint rm, guint add, gpointer p)
     {
       GValue value = G_VALUE_INIT;
       gtk_tree_model_get_value (priv->data_model, &iter, col, &value);
-      
+
       set_editor_widget_value (body, &value, GTK_EDITABLE (priv->editor));
       g_value_unset (&value);
     }
@@ -1795,6 +1799,7 @@ get_func (GtkClipboard *clipboard,
     }
 
   SswRange *source_range = g_object_get_data (G_OBJECT (clipboard), "source-range");
+  g_return_if_fail (source_range);
 
   GString *stuff = g_string_new ("");
   switch (info)
@@ -1832,9 +1837,16 @@ clear_func (GtkClipboard *clipboard,
 void
 ssw_sheet_body_set_clip (SswSheetBody *body, GtkClipboard *clip)
 {
+  PRIV_DECL (body);
   if (body == NULL)
     return;
 
+  SswRange *source_range = g_object_get_data (G_OBJECT (clip), "source-range");
+  g_free (source_range);
+  source_range = g_malloc (sizeof (*source_range));
+  g_object_set_data (G_OBJECT (clip), "source-range", source_range);
+  normalise_selection (priv->selection, source_range);
+
   if (!gtk_clipboard_set_with_owner (clip, targets, N_TARGETS,
 				     get_func, clear_func, G_OBJECT (body)))
     {
