@@ -860,12 +860,6 @@ ssw_sheet_set_clip (SswSheet *sheet, GtkClipboard *clip)
   if (!sheet->data_model)
     return;
 
-  SswRange *source_range = g_object_get_data (G_OBJECT (clip), "source-range");
-  g_free (source_range);
-  source_range = g_malloc (sizeof (*source_range));
-  g_object_set_data (G_OBJECT (clip), "source-range", source_range);
-  normalise_selection (sheet->selection, source_range);
-  
   ssw_sheet_body_set_clip (SSW_SHEET_BODY (sheet->selected_body), clip);
 }
 
@@ -1012,12 +1006,12 @@ target_marshaller (GtkClipboard *clip, GdkAtom *atoms, gint n_atoms,
 
   for (i = 0; i < n_atoms; ++i)
     {
-      if (atoms[i] == gdk_atom_intern ("text/html", TRUE))
+      if (atoms[i] == gdk_atom_intern_static_string ("text/html"))
 	{
 	  gtk_clipboard_request_contents (clip, atoms[i], html_parse, ps);
 	  break;
 	}
-      else if (atoms[i] == gdk_atom_intern ("UTF8_STRING", TRUE))
+      else if (atoms[i] == gdk_atom_intern_static_string ("UTF8_STRING"))
 	{
 	  gtk_clipboard_request_contents (clip, atoms[i], utf8_tab_delimited_parse, ps);
 	  break;
